'\"
'\" Generated from doc/xml/interface.n.xml
'\"
.so man.macros
.TH interface n "" Tcl ""
.BS
.SH NAME
interface \- how to use interfaces
.SH SYNOPSIS


.nf
\&\fBinterface\fP \fIoption\fR ?\fIarg arg ...\fR?
.fi
.BE
.SH DESCRIPTION
.PP
In Tcl "objects" can be created in many ways: in C, in Tcl or in any of the Tcl OO
extensions. The term object is used loosely here for each compound command: a command with
subcommands (methods) that encapsulates a certain functionality. Different kinds of objects
can often have similar functionality: eg. objects for connecting to different kinds
of databases would be nicely interchangeable if the would support the same set of methods
in the same way. In order to let objects easily communicate with each other they have to be
able to find out the others functionality.
.PP

An interface is a set of (related) methods that provide a predefined 
functionality.
An object implements an interface if it supports all the methods in the interface the proper way.
Of course, one object can support multiple interfaces. It must advertise the interfaces it 
supports via the "interface" method which has the following form

.CS
\&\fBpathName\fP interface ?\fIinterfaceName\fR?

.CE
which  will return a list of all interfaces supported if \fIinterfaceName\fR is not given.
If \fIinterfaceName\fR is given and the interface is supported, the supported version of the interface
is returned. If the interface is not supported, an error is returned.
.PP
An interface is basically defined by a description of the interface and its methods and a
test suite. This is implemented by an object in the interfaces namespace, that supports
the "interface" interface. (don't you just love self referential definitions?). This interface
defining object must have a name consisting of the interface name and its version number
seperated by a hyphen. It should have a doc method that returns the description of the
interface in XML format, and a test method that can be used to test a given object for
compliance. The test method takes the object to be tested as a first argument, optionally
followed by a number of options. These options depend completely on the interface being
defined, and can be used to provide necesary parameters or to allow the exlusion of certain tests.
.SH "THE INTERFACE COMMAND"
.PP
The \fBinterface\fP command is used to get information about interfaces.
It can have any of several forms, depending on the \fIoption\fR argument:
.TP
\&\fB\fBinterface\fP \fBlist\fP ?\fIpattern\fB?
\&\fRReturns a Tcl list whose elements are the available interfaces that match
\&\fIpattern\fR (using the matching rules of string match).
If pattern is omitted then the command returns all interfaces. The command returns
interfaces loaded in the interfaces namespaces, as well as autoloadable interfaces.
.TP
\&\fB\fBinterface\fP \fBversions\fP \fIinterfaceName\fB
\&\fRReturns a Tcl list whose elements are the available versions of the interface
given by \fIinterfaceName\fR.
.TP
\&\fB\fBinterface\fP \fBdoc\fP \fIinterfaceName\fB
\&\fRReturns the documentation of the interface given by \fIinterfaceName\fR. If
\&\fIinterfaceName\fR does not include a version number (of the form interface-1.0),
the documentation for the latest available version is returned.
.TP
\&\fB\fBinterface\fP \fBtest\fP \fIinterfaceName\fB \fIobjectName\fB ?options?
\&\fRtests whether the object \fIobjectName\fR fully supports the interface \fIinterfaceName\fR.
The command returns the number of the tests that failed. The actual errors made are sent to stdout,
and are available via "interface testsummarize". The options depend completely
on the interface being tested, and can be used to provide necesary parameters or to allow the
exlusion of certain tests.
.TP
\&\fB\fBinterface\fP \fBtestsummarize\fP
\&\fRReturns a summary of previous "interface test ..." runs, and returns an error if
some tests failed.
.TP
\&\fB\fBinterface\fP \fBtestclear\fP
\&\fRClears the results of previous calls to "interface test ...".
.SH "AN INTERFACE IMPLEMENTATION"
.PP
The interface package includes several Tcl commands to easily create an interface
defining object. Just use these as indicated in the following example:

.CS

proc ::interfaces::\fIinterfaceName\fR-\fIversion\fR {option args} {
 \fBinterface::implement\fP \fIinterfaceName\fR \fIinterfaceVersion\fR \fIoptions\fR $cmd $args
 \fBinterface::test\fP \fIdesription\fR \fIscript\fR \fIresult\fR ?\fIarg\fR ...?
 ...
 \fBinterface::testend\fP
}

.CE
.TP
\&\fB\fBinterface::implement\fP \fIinterfaceName\fB \fIinterfaceVersion\fB \fIdocfile\fB \fIoptions\fB \fIcmd\fB \fIargs\fB
\&\fRimplements the basic interface interface. It checks the method called in
\&\fIcmd\fR and returns the appropriate answers to the basic interface methods. If
\&\fIcmd\fR is test, the proc does not return, and the rest of the proc (that contains
the tests) is executed. If you want extra options besides the basic interface methods,
you should check for these before calling interface::implement. \fIcmd\fR must
be the first argument given to the procedure.
\&\fIinterfaceName\fR and \fIinterfaceVersion\fR are the name and the version of the
interface to be defined. They should be the same as the those used in the name of the
procedure.
\&\fIdocfile\fR contains the filename of the xml (tmml) documentation of the interface
\&\fIoptions\fR must be a list consisting of an even number of
elements. Each odd-numbered element in this list is treated as an option, and the
following element in list is used asdefault value for that option. The options
will be made available in the array opt. \fIargs\fR contains the resulting arguments
given to the proc, and should contain any options set by the caller.
.TP
\&\fB\fBinterface::test\fP \fIdesription\fB \fIscript\fB \fIresult\fB ?\fIarg\fB ...?
\&\fRTest one feature of the interface by running the code in script. This code will have access to the
variables
.RS
.TP
object
object being tested for complience to the interface
.TP
opt
array containing options given to the proc
.TP
interface
name of interface being tested
.TP
version
version of interface being tested
.PP
Some extra arguments \fIarg\fR can be given with the following meaning:
.TP
error
The test script must cause an error (\fIresult\fR must
match the error message returned)
.TP
skipon \fIcondition\fR
The test will be skipped if \fIcondition\fR is true
.TP
match
use string match instead of == to check whether the
pattern \fIresult\fR and the result returned by \fIscript\fR match
.TP
regexp
use regexp instead of == to check whether the pattern
\&\fIresult\fR and the result returned by \fIscript\fR match
.TP
interface
name of interface being tested
.TP
version
version of interface being tested
.RE
.TP
\&\fB\fBinterface::testsummarize\fP
\&\fRshould go at the end of the defining proc. Returns an error and an appropriate
error message if there were any errors.
.SH "SEE ALSO"
interface_interface
.SH KEYWORDS
interface, interface::interface, interface::implement, interface::test, interface::testsummarize
